@@ -1,4 +1,4 @@
-# $Id: NodeList.pm 422 2002-11-08 17:10:30Z phish $
+# $Id: NodeList.pm 603 2006-09-23 21:47:38Z pajas $
 
 package XML::LibXML::NodeList;
 use strict;
@@ -6,6 +6,9 @@ use XML::LibXML::Boolean;
 use XML::LibXML::Literal;
 use XML::LibXML::Number;
 
+use vars qw ($VERSION);
+  $VERSION = "1.61"; # VERSION TEMPLATE: DO NOT CHANGE
+
 use overload 
 		'""' => \&to_literal,
                 'bool' => \&to_boolean,
@@ -16,6 +19,11 @@ sub new {
 	bless [@_], $class;
 }
 
+sub new_from_ref {
+	my ($class,$array_ref,$reuse) = @_;
+	return bless $reuse ? $array_ref : [@$array_ref], $class;
+}
+
 sub pop {
 	my $self = CORE::shift;
 	CORE::pop @$self;
