@@ -1,33 +1,23 @@
 # $Id: XPathContext.pm 422 2002-11-08 17:10:30Z phish $
-#
-# This is free software, you may use it and distribute it under the same terms as
-# Perl itself.
-#
-# Copyright 2001-2003 AxKit.com Ltd., 2002-2006 Christian Glahn, 2006-2009 Petr Pajas
-#
-#
 
 package XML::LibXML::XPathContext;
 
 use strict;
 use vars qw($VERSION @ISA $USE_LIBXML_DATA_TYPES);
 
-use Carp;
 use XML::LibXML;
 use XML::LibXML::NodeList;
 
-$VERSION = "1.69_2"; # VERSION TEMPLATE: DO NOT CHANGE
+  $VERSION = "1.61"; # VERSION TEMPLATE: DO NOT CHANGE
 
 # should LibXML XPath data types be used for simple objects
 # when passing parameters to extension functions (default: no)
 $USE_LIBXML_DATA_TYPES = 0;
 
-sub CLONE_SKIP { 1 }
-
 sub findnodes {
     my ($self, $xpath, $node) = @_;
 
-    my @nodes = $self->_guarded_find_call('_findnodes', $node, $xpath);
+    my @nodes = $self->_guarded_find_call('_findnodes', $xpath, $node);
 
     if (wantarray) {
         return @nodes;
@@ -40,7 +30,7 @@ sub findnodes {
 sub find {
     my ($self, $xpath, $node) = @_;
 
-    my ($type, @params) = $self->_guarded_find_call('_find', $node, $xpath,0);
+    my ($type, @params) = $self->_guarded_find_call('_find', $xpath, $node);
 
     if ($type) {
         return $type->new(@params);
@@ -48,19 +38,13 @@ sub find {
     return undef;
 }
 
-sub exists {
-    my ($self, $xpath, $node) = @_;
-    my (undef, $value) = $self->_guarded_find_call('_find', $node, $xpath,1);
-    return $value;
-}
-
 sub findvalue {
     my $self = shift;
     return $self->find(@_)->to_literal->value;
 }
 
 sub _guarded_find_call {
-    my ($self, $method, $node)=(shift,shift,shift);
+    my ($self, $method, $xpath, $node) = @_;
 
     my $prev_node;
     if (ref($node)) {
@@ -69,16 +53,12 @@ sub _guarded_find_call {
     }
     my @ret;
     eval {
-        @ret = $self->$method(@_);
+        @ret = $self->$method($xpath);
     };
     $self->_free_node_pool;
     $self->setContextNode($prev_node) if ref($node);
 
-    if ($@) { 
-      my $err = $@;
-      chomp $err;
-      croak $err; 
-    }
+    if ($@) { die "ERROR: $@"; }
 
     return @ret;
 }
